############################################################################
# Copyright (c) 2020 SIOS Technology Corp.
#ident "@(#):quickchk.ksh 7.1.1.1"
# 
# FileName     :  quickchk.ksh
# Description  :  Quickcheck script for SQL Server Recovery Kit
# 
# args:	-t TAG		Tag name of resource to monitor.
#		-i ID		ID of resource to monitor.
#
# Valid error codes for this file are 14975 - 14999
# 
# Initial Implementation Date:      7/24/97
# 
# 
# CHG #      DESCRIPTION OF CHANGE               CHANGED BY       DATE
# =========================================================================
# 0001      Added support for SQL 7.0		        meh		  10/20/98                      
# 0002 lknt000091 Changing copyright notice        Tarua      09/18/2000
#	               -from NCR to SteelEye 
# 0003	Changes for multiple instances		PEB		11/19/2001
#		support for ADHLPR service, 4.0
#		GUI lkwin00000297
# 0004  lkwin00000599 - If MSSQLServer paused,    SFH           02/26/2002
#       - log error and abort with exit 0.
# 0005  lkwin00001203 - Quote tag & system names  lcg           08/05/2003
# 0006  lkwin00002061 - Remove support for SQL 7  CMR           02/17/2006
# 0007  lkwin00002163 - Add support for SQL 9     CMR           02/17/2006
# 0008  lkwin00002180 - Add CMD variable for msg  CMR           03/22/2006
# 0009  bug953        - Replacement of sc query   Aki           03/18/2010
#                       that language influences
# 0010  bug4272       - Don't fail the resource   DL            12/20/2017
#                       if ins_list fails
# 0011  PW-2505       - Validate service state to DK            07/22/2020
#                       avoid false service falures 
#
#############################################################################

FAILURE=1
SUCCESS=0

if [ "$LKROOT" = "" ]
then
	echo "Lifekeeper Root Directory Environment parameter LKROOT not set."  >&2
	exit $SUCCESS
fi
. $LKROOT/Admin/kit/sqlapp/bin/env_sql.ksh >NUL:
CMD=`basename $0 2>NUL:`
# set -x
while getopts :t:i: o
do
	case $o in
	t)	TAG="$OPTARG";;
	i)	ID="$OPTARG";;
	[?])	;;
	esac
done

if [ "$TAG" = "" ]
then
	env_logerror $CMD 'tagusage' 'TO_STDERR' 'FRS_ERR'
	exit $SUCCESS
fi
if [ "$ID" = "" ]
then
	env_logerror $CMD 'idusage' 'TO_STDERR' 'FRS_ERR'
	exit $SUCCESS
fi

# 0011 Use local function to obtain and validate service state
# Service states: STOPPED=1, START_PENDING=2, STOP_PENDING=3,
#  RUNNING=4, CONTINUE_PENDING=5, PAUSE_PENDING=6, PAUSED=7
GetSvcState(){
	# Calling syntax: GetSvcState <service name>
	# The scQueryState.exe can fail. On error save error
	# message in the state variable so it can be logged.
	state=`$LKBIN/ScQueryState "$1" 2>&1`
	if [ $? -ne 0 ]
	then
		# Query to Service Control Manager has failed.
		# Log as a warning and include the scQueryState error message		
		env_logerror $CMD 'servicestateinvalid' 'TO_STDERR' 'FRS_WARN' "$1" "$state"
		exit $SUCCESS
	fi
	# Parse the service state string down to a single character, value 1-7
	state=`echo $state | $LKBIN/awk '{if($1 == "STATE:") print $2}' 2>NUL:`
	# Validate the state code
	if ! echo $state | egrep -q '^[1-7]{1}$'
	then
		# Log invalid code warning and include the code
		env_logerror $CMD 'servicestateinvalid' 'TO_STDERR' 'FRS_WARN' "$1" "$state"
		exit $SUCCESS
	fi
}

# 0003
# Get the name for the MSSQL server to check
SYS=`$LKROOT/bin/lcduname`
NAMECHECK=`$LKROOT/bin/ins_list -t "$TAG" | cut -d"" -f6 | cut -d"" -f2 2>NUL:`
# 0010 begin
# If ins_list fails or SYS is empty, log a warning message and don't fail the resource
if [ "$NAMECHECK" = "" -o "$SYS" = "" ]
then
        env_logerror $CMD 'noinfo' 'TO_STDERR' 'FRS_WARN' "$TAG"
        exit $SUCCESS 
fi
# 0010 end
# call function to translate instance name into
# all corresponding connect, display, and management
# strings.  This will set LKSQLSERVER LKSQLSYSINS LKSQLAGENT 
# LKSQLFULLTEXT LKSQLREPSERVER LKSQLSERVER 
# LKSQLDSPSVC LKSQLOLAP 
env_gettranslation $NAMECHECK $SYS $SYS

# Check the state of the MSSQLServer service.
# 0011 Use local function to obtain and validate service state
GetSvcState $LKSQLSERVER

# 0004 If MSSQLServer in a "paused" state, log error requiring user intervention and exit
#      quick check with an exit 0.
if [ "$state" = "$SVCPAUSED" ]
then
	env_logerror $CMD 'paused' 'TO_STDERR' 'FRS_ERR'
	exit 0
fi

# If it is not running then fail the check.
if [ "$state" != "$SVCRUNNING" ]
then
	# 0011 Include actual service state in error message
	env_logerror $CMD 'notrunning' 'TO_STDERR' 'FRS_ERR' 'quickchk' "$LKSQLSERVER(state=$state)"
	exit $FAILURE
fi

#
# Get the instance specific information
#

# 0003
ADDSVC=`env_getproclist $TAG 2>NUL:`
for SRVC in $ADDSVC
do
	# 0011 Use local function to obtain and validate service state
	GetSvcState $SRVC
	if [ "$state" != "$SVCRUNNING" ]
	then
		# 0011 Include actual service state in error message
		env_logerror $CMD 'notrunning' 'TO_STDERR' 'FRS_ERR' 'quickchk' "$SRVC(state=$state)"
		exit $FAILURE
	fi
done
exit $SUCCESS
